@@ -20,14 +20,17 @@
 #ifndef AKONADI_MONITOR_P_H
 #define AKONADI_MONITOR_P_H
 
+#include "akonadiprivate_export.h"
 #include "monitor.h"
 #include "collection.h"
 #include "collectionstatisticsjob.h"
+#include "collectionfetchscope.h"
 #include "item.h"
 #include "itemfetchscope.h"
 #include "job.h"
 #include <akonadi/private/notificationmessage_p.h>
 #include "notificationmanagerinterface.h"
+#include "entitycache_p.h"
 
 #include <kmimetype.h>
 
@@ -41,11 +44,12 @@ class Monitor;
 /**
  * @internal
  */
-class MonitorPrivate
+class AKONADI_TESTS_EXPORT MonitorPrivate
 {
   public:
     MonitorPrivate( Monitor *parent );
     virtual ~MonitorPrivate() {}
+    void init();
 
     Monitor *q_ptr;
     Q_DECLARE_PUBLIC( Monitor )
@@ -57,7 +61,14 @@ class MonitorPrivate
     bool monitorAll;
     QList<QByteArray> sessions;
     ItemFetchScope mItemFetchScope;
-    QHash<KJob*,NotificationMessage> pendingJobs;
+    CollectionFetchScope mCollectionFetchScope;
+    Session *session;
+    CollectionCache collectionCache;
+    ItemCache itemCache;
+    QQueue<NotificationMessage> pendingNotifications;
+    QQueue<NotificationMessage> pipeline;
+    bool fetchCollection;
+    bool fetchCollectionStatistics;
 
     bool isCollectionMonitored( Collection::Id collection, const QByteArray &resource ) const
     {
@@ -81,31 +92,112 @@ class MonitorPrivate
       return sessions.contains( sessionId );
     }
 
-    bool connectToNotificationManager();
+    bool isMoveDestinationResourceMonitored( const NotificationMessage &msg )
+    {
+      if ( msg.operation() != NotificationMessage::Move || msg.itemParts().isEmpty() )
+        return false;
+      const QByteArray res = *(msg.itemParts().begin());
+      return resources.contains( res );
+    }
+
+    // Virtual so it can be overridden in FakeMonitor.
+    virtual bool connectToNotificationManager();
     bool acceptNotification( const NotificationMessage &msg );
-    bool processNotification( const NotificationMessage &msg );
+    void dispatchNotifications();
+
+    // Called when the monitored item/collection changes, checks if the queued messages
+    // are still accepted, if not they are removed
+    void cleanOldNotifications();
+
+    bool ensureDataAvailable( const NotificationMessage &msg );
+    void emitNotification( const NotificationMessage &msg );
+    void updatePendingStatistics( const NotificationMessage &msg );
+    void invalidateCaches( const NotificationMessage &msg );
+
+    /** Used by ResourceBase to inform us about collection changes before the notifications are emitted,
+        needed to avoid the missing RID race on change replay.
+    */
+    void invalidateCache( const Collection &col );
+
+    virtual int pipelineSize() const;
 
     // private slots
-    void sessionDestroyed( QObject* );
+    void dataAvailable();
+    void slotSessionDestroyed( QObject* );
     void slotStatisticsChangedFinished( KJob* );
     void slotFlushRecentlyChangedCollections();
 
+    void appendAndCompress( const NotificationMessage &msg  );
+
     virtual void slotNotify( const NotificationMessage::List &msgs );
-    void slotItemJobFinished( KJob* job );
-    void slotCollectionJobFinished( KJob *job );
 
     void emitItemNotification( const NotificationMessage &msg, const Item &item = Item(),
                                const Collection &collection = Collection(), const Collection &collectionDest = Collection() );
     void emitCollectionNotification( const NotificationMessage &msg, const Collection &col = Collection(),
-                                     const Collection &par = Collection() );
+                                     const Collection &par = Collection(), const Collection &dest = Collection() );
 
-    bool fetchCollection;
-    bool fetchCollectionStatistics;
+
+    /**
+      @brief Class used to determine when to purge items in a Collection
+
+      The buffer method can be used to buffer a Collection. This may cause another Collection
+      to be purged if it is removed from the buffer.
+
+      The purge method is used to purge a Collection from the buffer, but not the model.
+      This is used for example, to not buffer Collections anymore if they get referenced,
+      and to ensure that one Collection does not appear twice in the buffer.
+
+      Check whether a Collection is buffered using the isBuffered method.
+    */
+    class PurgeBuffer
+    {
+      // Buffer the most recent 10 unreferenced Collections
+      static const int MAXBUFFERSIZE = 10;
+    public:
+      explicit PurgeBuffer()
+        : m_index( 0 ),
+          m_bufferSize( MAXBUFFERSIZE )
+      {
+      }
+
+      /**
+        Adds @p id to the Collections to be buffered
+
+        @returns The collection id which was removed form the buffer or -1 if none.
+      */
+      Collection::Id buffer( Collection::Id id );
+
+      /**
+      Removes @p id from the Collections being buffered
+      */
+      void purge( Collection::Id id );
+
+      bool isBuffered( Collection::Id id ) const
+      {
+        return m_buffer.contains( id );
+      }
+
+    private:
+      QList<Collection::Id> m_buffer;
+      int m_index;
+      int m_bufferSize;
+    } m_buffer;
+
+
+    QHash<Collection::Id, int> refCountMap;
+    bool useRefCounting;
+    void ref( Collection::Id id );
+    Collection::Id deref( Collection::Id id );
 
   private:
     // collections that need a statistics update
     QSet<Collection::Id> recentlyChangedCollections;
 
+    /**
+      @returns True if @p msg should be ignored. Otherwise appropriate signals are emitted for it.
+    */
+    bool isLazilyIgnored( const NotificationMessage & msg ) const;
+
     bool isCollectionMonitored( Collection::Id collection ) const
     {
       if ( collections.contains( Collection( collection ) ) )
@@ -134,59 +226,20 @@ class MonitorPrivate
 
     void fetchStatistics( Collection::Id colId )
     {
-      CollectionStatisticsJob *job = new CollectionStatisticsJob( Collection( colId ), q_ptr );
-      QObject::connect( job, SIGNAL(result(KJob*)), q_ptr, SLOT(slotStatisticsChangedFinished(KJob*)) );
+      CollectionStatisticsJob *job = new CollectionStatisticsJob( Collection( colId ), session );
+      QObject::connect( job, SIGNAL( result( KJob* ) ), q_ptr, SLOT( slotStatisticsChangedFinished( KJob* ) ) );
     }
 
     void notifyCollectionStatisticsWatchers( Collection::Id collection, const QByteArray &resource )
     {
-      if ( isCollectionMonitored( collection, resource ) ) {
+      if ( collection > 0 && isCollectionMonitored( collection, resource ) ) {
         if (recentlyChangedCollections.empty() )
-          QTimer::singleShot( 500, q_ptr, SLOT(slotFlushRecentlyChangedCollections()) );
+          QTimer::singleShot( 500, q_ptr, SLOT( slotFlushRecentlyChangedCollections() ) );
         recentlyChangedCollections.insert( collection );
       }
     }
 };
 
-
-/**
- * @internal
- *
- * A job which fetches an item and a collection.
- */
-class AKONADI_EXPORT ItemCollectionFetchJob : public Job
-{
-  Q_OBJECT
-
-  public:
-    explicit ItemCollectionFetchJob( const Item &item, Collection::Id collectionId, Collection::Id destCollectionId, QObject *parent = 0 );
-    ~ItemCollectionFetchJob();
-
-    Item item() const;
-    Collection collection() const;
-    Collection destCollection() const;
-
-    void setFetchScope( const ItemFetchScope &fetchScope );
-
-  protected:
-    virtual void doStart();
-
-  private Q_SLOTS:
-    void collectionJobDone( KJob* job );
-    void destCollectionJobDone( KJob* job );
-    void itemJobDone( KJob* job );
-
-  private:
-    Item mReferenceItem;
-    Collection::Id mCollectionId;
-    Collection::Id mDestCollectionId;
-
-    Item mItem;
-    Collection mCollection;
-    Collection mDestCollection;
-    ItemFetchScope mFetchScope;
-};
-
 }
 
 #endif
