@@ -20,6 +20,7 @@
 #include "resourcescheduler_p.h"
 
 #include <kdebug.h>
+#include <klocale.h>
 
 #include <QtCore/QTimer>
 #include <QtDBus/QDBusInterface>
@@ -43,9 +44,10 @@ void ResourceScheduler::scheduleFullSync()
 {
   Task t;
   t.type = SyncAll;
-  if ( !mTaskList.isEmpty() && ( mTaskList.last() == t || mCurrentTask == t ) )
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
     return;
-  mTaskList << t;
+  queue << t;
   signalTaskToTracker( t, "SyncAll" );
   scheduleNext();
 }
@@ -54,9 +56,10 @@ void ResourceScheduler::scheduleCollectionTreeSync()
 {
   Task t;
   t.type = SyncCollectionTree;
-  if ( !mTaskList.isEmpty() && ( mTaskList.last() == t || mCurrentTask == t ) )
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
     return;
-  mTaskList << t;
+  queue << t;
   signalTaskToTracker( t, "SyncCollectionTree" );
   scheduleNext();
 }
@@ -66,9 +69,10 @@ void ResourceScheduler::scheduleSync(const Collection & col)
   Task t;
   t.type = SyncCollection;
   t.collection = col;
-  if ( !mTaskList.isEmpty() && ( mTaskList.last() == t || mCurrentTask == t ) )
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
     return;
-  mTaskList << t;
+  queue << t;
   signalTaskToTracker( t, "SyncCollection" );
   scheduleNext();
 }
@@ -79,10 +83,24 @@ void ResourceScheduler::scheduleItemFetch(const Item & item, const QSet<QByteArr
   t.type = FetchItem;
   t.item = item;
   t.itemParts = parts;
-  t.dbusMsg = msg;
-  if ( !mTaskList.isEmpty() && ( mTaskList.last() == t || mCurrentTask == t ) )
+
+  // if the current task does already fetch the requested item, break here but
+  // keep the dbus message, so we can send the reply later on
+  if ( mCurrentTask == t ) {
+    mCurrentTask.dbusMsgs << msg;
     return;
-  mTaskList << t;
+  }
+
+  // If this task is already in the queue, merge with it.
+  TaskList& queue = queueForTaskType( t.type );
+  const int idx = queue.indexOf( t );
+  if ( idx != -1 ) {
+    queue[ idx ].dbusMsgs << msg;
+    return;
+  }
+
+  t.dbusMsgs << msg;
+  queue << t;
   signalTaskToTracker( t, "FetchItem" );
   scheduleNext();
 }
@@ -91,9 +109,10 @@ void ResourceScheduler::scheduleResourceCollectionDeletion()
 {
   Task t;
   t.type = DeleteResourceCollection;
-  if ( !mTaskList.isEmpty() && ( mTaskList.last() == t || mCurrentTask == t ) )
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
     return;
-  mTaskList << t;
+  queue << t;
   signalTaskToTracker( t, "DeleteResourceCollection" );
   scheduleNext();
 }
@@ -102,12 +121,11 @@ void ResourceScheduler::scheduleChangeReplay()
 {
   Task t;
   t.type = ChangeReplay;
-  if ( mTaskList.contains( t ) )
+  TaskList& queue = queueForTaskType( t.type );
+  // see ResourceBase::changeProcessed() for why we do not check for mCurrentTask == t here like in the other tasks
+  if ( queue.contains( t ) )
     return;
-  // change replays have to happen before we pull changes from the backend, otherwise
-  // we will overwrite our still unsaved local changes if the backend can't do
-  // incremental retrieval
-  mTaskList.prepend( t );
+  queue << t;
   signalTaskToTracker( t, "ChangeReplay" );
   scheduleNext();
 }
@@ -116,7 +134,9 @@ void Akonadi::ResourceScheduler::scheduleFullSyncCompletion()
 {
   Task t;
   t.type = SyncAllDone;
-  mTaskList << t;
+  TaskList& queue = queueForTaskType( t.type );
+  // no compression here, all this does is emitting a D-Bus signal anyway, and compression can trigger races on the receiver side with the signal being lost
+  queue << t;
   signalTaskToTracker( t, "SyncAllDone" );
   scheduleNext();
 }
@@ -128,30 +148,21 @@ void Akonadi::ResourceScheduler::scheduleCustomTask( QObject *receiver, const ch
   t.receiver = receiver;
   t.methodName = methodName;
   t.argument = argument;
-  if ( mTaskList.contains( t ) )
+  QueueType queueType = GenericTaskQueue;
+  if ( priority == ResourceBase::AfterChangeReplay )
+    queueType = AfterChangeReplayQueue;
+  TaskList& queue = mTaskList[ queueType ];
+
+  if ( queue.contains( t ) )
     return;
 
   switch (priority) {
   case ResourceBase::Prepend:
-    mTaskList.prepend(t);
-    break;
-  case ResourceBase::AfterChangeReplay:
-    {
-      QMutableListIterator<Task> it(mTaskList);
-      bool inserted = false;
-      while (it.hasNext() && !inserted) {
-        if (it.next().type != ChangeReplay) {
-          it.previous(); // next returns the item *and* advances the iterator.
-          it.insert(t);
-          inserted = true;
-        }
-      }
-      if (!inserted)
-        mTaskList.append(t);
-    }
+    queue.prepend( t );
     break;
   default:
-    mTaskList.append(t);
+    queue.append(t);
+    break;
   }
 
   signalTaskToTracker( t, "Custom-" + t.methodName );
@@ -161,13 +172,13 @@ void Akonadi::ResourceScheduler::scheduleCustomTask( QObject *receiver, const ch
 void ResourceScheduler::taskDone()
 {
   if ( isEmpty() )
-    emit status( AgentBase::Idle );
+    emit status( AgentBase::Idle, i18nc( "@info:status Application ready for work", "Ready" ) );
 
   if ( s_resourcetracker ) {
     QList<QVariant> argumentList;
     argumentList << QString::number( mCurrentTask.serial )
                  << QString();
-    s_resourcetracker->asyncCallWithArgumentList(QLatin1String("jobEnded"), argumentList);
+    s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobEnded" ), argumentList);
   }
 
   mCurrentTask = Task();
@@ -180,12 +191,12 @@ void ResourceScheduler::deferTask()
     QList<QVariant> argumentList;
     argumentList << QString::number( mCurrentTask.serial )
                  << QString();
-    s_resourcetracker->asyncCallWithArgumentList(QLatin1String("jobEnded"), argumentList);
+    s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobEnded" ), argumentList);
   }
 
   Task t = mCurrentTask;
   mCurrentTask = Task();
-  mTaskList << t;
+  mTaskList[GenericTaskQueue] << t;
   signalTaskToTracker( t, "DeferedTask" );
 
   scheduleNext();
@@ -193,27 +204,36 @@ void ResourceScheduler::deferTask()
 
 bool ResourceScheduler::isEmpty()
 {
-  return mTaskList.isEmpty();
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    if ( !mTaskList[i].isEmpty() )
+      return false;
+  }
+  return true;
 }
 
 void ResourceScheduler::scheduleNext()
 {
-  if ( mCurrentTask.type != Invalid || mTaskList.isEmpty() || !mOnline )
+  if ( mCurrentTask.type != Invalid || isEmpty() || !mOnline )
     return;
-  QTimer::singleShot( 0, this, SLOT(executeNext()) );
+  QTimer::singleShot( 0, this, SLOT( executeNext() ) );
 }
 
 void ResourceScheduler::executeNext()
 {
-  if( mCurrentTask.type != Invalid || mTaskList.isEmpty() )
+  if ( mCurrentTask.type != Invalid || isEmpty() )
     return;
 
-  mCurrentTask = mTaskList.takeFirst();
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    if ( !mTaskList[ i ].isEmpty() ) {
+      mCurrentTask = mTaskList[ i ].takeFirst();
+      break;
+    }
+  }
 
   if ( s_resourcetracker ) {
     QList<QVariant> argumentList;
     argumentList << QString::number( mCurrentTask.serial );
-    s_resourcetracker->asyncCallWithArgumentList(QLatin1String("jobStarted"), argumentList);
+    s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobStarted" ), argumentList);
   }
 
   switch ( mCurrentTask.type ) {
@@ -240,7 +260,7 @@ void ResourceScheduler::executeNext()
       break;
     case Custom:
     {
-      bool success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName, Q_ARG(QVariant, mCurrentTask.argument) );;
+      bool success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName, Q_ARG(QVariant, mCurrentTask.argument) );
       if ( !success )
         success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName );
 
@@ -248,8 +268,11 @@ void ResourceScheduler::executeNext()
         kError() << "Could not invoke slot" << mCurrentTask.methodName << "on" << mCurrentTask.receiver << "with argument" << mCurrentTask.argument;
       break;
     }
-    default:
+    default: {
+      kError() << "Unhandled task type" << mCurrentTask.type;
+      dump();
       Q_ASSERT( false );
+    }
   }
 }
 
@@ -265,20 +288,38 @@ void ResourceScheduler::setOnline(bool state)
   mOnline = state;
   if ( mOnline ) {
     scheduleNext();
-  } else if ( mCurrentTask.type != Invalid ) {
-    // abort running task
-    mTaskList.prepend( mCurrentTask );
-    mCurrentTask = Task();
+  } else {
+    if ( mCurrentTask.type != Invalid ) {
+      // abort running task
+      queueForTaskType( mCurrentTask.type ).prepend( mCurrentTask );
+      mCurrentTask = Task();
+    }
+    // abort pending synchronous tasks, might take longer until the resource goes online again
+    TaskList& itemFetchQueue = queueForTaskType( FetchItem );
+    for ( QList< Task >::iterator it = itemFetchQueue.begin(); it != itemFetchQueue.end(); ) {
+      if ( (*it).type == FetchItem ) {
+        (*it).sendDBusReplies( false );
+        it = itemFetchQueue.erase( it );
+        if ( s_resourcetracker ) {
+          QList<QVariant> argumentList;
+          argumentList << QString::number( mCurrentTask.serial )
+                       << QLatin1String( "Job canceled." );
+          s_resourcetracker->asyncCallWithArgumentList( QLatin1String( "jobEnded" ), argumentList );
+        }
+      } else {
+        ++it;
+      }
+    }
   }
 }
 
 void ResourceScheduler::signalTaskToTracker( const Task &task, const QByteArray &taskType )
 {
   // if there's a job tracer running, tell it about the new job
-  if ( !s_resourcetracker && QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String("org.kde.akonadiconsole") ) ) {
-    s_resourcetracker = new QDBusInterface( QLatin1String("org.kde.akonadiconsole"),
-                                       QLatin1String("/resourcesJobtracker"),
-                                       QLatin1String("org.freedesktop.Akonadi.JobTracker"),
+  if ( !s_resourcetracker && QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
+    s_resourcetracker = new QDBusInterface( QLatin1String( "org.kde.akonadiconsole" ),
+                                       QLatin1String( "/resourcesJobtracker" ),
+                                       QLatin1String( "org.freedesktop.Akonadi.JobTracker" ),
                                        QDBusConnection::sessionBus(), 0 );
   }
 
@@ -288,7 +329,7 @@ void ResourceScheduler::signalTaskToTracker( const Task &task, const QByteArray
                  << QString::number( task.serial )
                  << QString()
                  << QString::fromLatin1( taskType );
-    s_resourcetracker->asyncCallWithArgumentList(QLatin1String("jobCreated"), argumentList);
+    s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobCreated" ), argumentList);
   }
 }
 
@@ -296,15 +337,92 @@ void ResourceScheduler::collectionRemoved( const Akonadi::Collection &collection
 {
   if ( !collection.isValid() ) // should not happen, but you never know...
     return;
-  for ( QList<Task>::iterator it = mTaskList.begin(); it != mTaskList.end(); ) {
+  TaskList& queue = queueForTaskType( SyncCollection );
+  for ( QList<Task>::iterator it = queue.begin(); it != queue.end(); ) {
     if ( (*it).type == SyncCollection && (*it).collection == collection ) {
-      it = mTaskList.erase( it );
+      it = queue.erase( it );
       kDebug() << " erasing";
     } else
       ++it;
   }
 }
 
+void ResourceScheduler::Task::sendDBusReplies( bool success )
+{
+  Q_FOREACH( const QDBusMessage &msg, dbusMsgs ) {
+    QDBusMessage reply( msg );
+    reply << success;
+    QDBusConnection::sessionBus().send( reply );
+  }
+}
+
+ResourceScheduler::QueueType ResourceScheduler::queueTypeForTaskType( TaskType type )
+{
+  switch( type ) {
+  case ChangeReplay:
+    return ChangeReplayQueue;
+  case FetchItem:
+    return ItemFetchQueue;
+  default:
+    return GenericTaskQueue;
+  }
+}
+
+ResourceScheduler::TaskList& ResourceScheduler::queueForTaskType( TaskType type )
+{
+  const QueueType qt = queueTypeForTaskType( type );
+  return mTaskList[ qt ];
+}
+
+void ResourceScheduler::dump()
+{
+  kDebug() << "ResourceScheduler: Online:" << mOnline;
+  kDebug() << " current task:" << mCurrentTask;
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    const TaskList& queue = mTaskList[i];
+    kDebug() << " queue" << i << queue.size() << "tasks:";
+    for ( QList<Task>::const_iterator it = queue.begin(); it != queue.end(); ++it ) {
+      kDebug() << "  " << (*it);
+    }
+  }
+}
+
+void ResourceScheduler::clear()
+{
+  kDebug() << "Clearing ResourceScheduler queues:";
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    TaskList& queue = mTaskList[i];
+    queue.clear();
+  }
+  mCurrentTask = Task();
+}
+
+static const char s_taskTypes[][25] = {
+      "Invalid",
+      "SyncAll",
+      "SyncCollectionTree",
+      "SyncCollection",
+      "FetchItem",
+      "ChangeReplay",
+      "DeleteResourceCollection",
+      "SyncAllDone",
+      "Custom"
+};
+
+QDebug Akonadi::operator<<( QDebug d, const ResourceScheduler::Task& task )
+{
+  d << task.serial << s_taskTypes[task.type];
+  if ( task.type != ResourceScheduler::Invalid ) {
+    if ( task.collection.id() != -1 )
+      d << "collection" << task.collection.id();
+    if ( task.item.id() != -1 )
+      d << "item" << task.item.id();
+    if ( !task.methodName.isEmpty() )
+      d << task.methodName << task.argument;
+  }
+  return d;
+}
+
 //@endcond
 
 #include "resourcescheduler_p.moc"
