@@ -67,11 +67,13 @@ class ResourceScheduler : public QObject
         Collection collection;
         Item item;
         QSet<QByteArray> itemParts;
-        QDBusMessage dbusMsg;
+        QList<QDBusMessage> dbusMsgs;
         QObject *receiver;
         QByteArray methodName;
         QVariant argument;
 
+        void sendDBusReplies( bool success );
+
         bool operator==( const Task &other ) const
         {
           return type == other.type
@@ -142,6 +144,18 @@ class ResourceScheduler : public QObject
     */
     void setOnline( bool state );
 
+    /**
+       Print debug output showing the state of the scheduler.
+    */
+    void dump();
+
+    /**
+       Clear the state of the scheduler. Warning: this is intended to be
+       used purely in debugging scenarios, as it might cause loss of uncommitted
+       local changes.
+    */
+    void clear();
+
   public Q_SLOTS:
     /**
       Schedules replaying changes.
@@ -180,11 +194,35 @@ class ResourceScheduler : public QObject
   private:
     void signalTaskToTracker( const Task &task, const QByteArray &taskType );
 
-    QList<Task> mTaskList;
+    // We have a number of task queues, by order of priority.
+    // * ChangeReplay must be first:
+    //    change replays have to happen before we pull changes from the backend, otherwise
+    //    we will overwrite our still unsaved local changes if the backend can't do
+    //    incremental retrieval
+    //
+    // * then the stuff that is "immediately after change replay", like writeFile calls.
+    // * then ItemFetch tasks, because they are made by blocking DBus calls
+    // * then everything else.
+    enum QueueType {
+      ChangeReplayQueue, // one task at most
+      AfterChangeReplayQueue, // also one task at most, currently
+      ItemFetchQueue,
+      GenericTaskQueue,
+      NQueueCount
+    };
+    typedef QList<Task> TaskList;
+
+    static QueueType queueTypeForTaskType( TaskType type );
+    TaskList& queueForTaskType( TaskType type );
+
+    TaskList mTaskList[ NQueueCount ];
+
     Task mCurrentTask;
     bool mOnline;
 };
 
+QDebug operator<<( QDebug, const ResourceScheduler::Task& task );
+
 //@endcond
 
 }
