@@ -15,8 +15,9 @@
  * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
+#include "config.h" //krazy:exclude=includes
 
+#include <KDE/KDebug>
 #include <kstandarddirs.h>
 
 #include <QtCore/QDir>
@@ -62,7 +63,7 @@ void Config::readConfiguration(const QString &configfile)
   if ( root.tagName() != "config" )
     qFatal( "could not file root tag" );
 
-  const QString basePath = QFileInfo( configfile ).absolutePath() + "/";
+  const QString basePath = QFileInfo( configfile ).absolutePath() + '/';
 
   QDomNode node = root.firstChild();
   while ( !node.isNull() ) {
@@ -76,6 +77,13 @@ void Config::readConfiguration(const QString &configfile)
         setXdgDataHome( basePath + element.text() );
       } else if ( element.tagName() == "agent" ) {
         insertAgent( element.text(), element.attribute( "synchronize", "false" ) == QLatin1String("true") );
+      } else if ( element.tagName() == "envvar" ) {
+        const QString name = element.attribute( "name" );
+        if( name.isEmpty() ) {
+          kWarning() << "Given envvar with no name.";
+        } else {
+          mEnvVars[ name ] = element.text();
+        }
       }
     }
 
@@ -125,3 +133,8 @@ QList<QPair<QString, bool> > Config::agents() const
 {
   return mAgents;
 }
+
+QHash<QString, QString> Config::envVars() const
+{
+  return mEnvVars;
+}
