@@ -75,6 +75,9 @@ class KCal::Calendar::Private
     ~Private()
     {
       delete mTimeZones;
+      if ( mFilter != mDefaultFilter ) {
+        delete mFilter;
+      }
       delete mDefaultFilter;
     }
     KDateTime::Spec timeZoneIdSpec( const QString &timeZoneId, bool view );
@@ -291,7 +294,7 @@ Event::List Calendar::sortEvents( Event::List *eventList,
                                   SortDirection sortDirection )
 {
   Event::List eventListSorted;
-  Event::List tempList, t;
+  Event::List tempList;
   Event::List alphaList;
   Event::List::Iterator sortIt;
   Event::List::Iterator eit;
@@ -326,11 +329,11 @@ Event::List Calendar::sortEvents( Event::List *eventList,
       eventListSorted.insert( sortIt, *eit );
     }
     if ( sortDirection == SortDirectionAscending ) {
-      // Prepend the list of Events without End DateTimes
+      // Prepend the list of all-day Events
       tempList += eventListSorted;
       eventListSorted = tempList;
     } else {
-      // Append the list of Events without End DateTimes
+      // Append the list of all-day Events
       eventListSorted += tempList;
     }
     break;
@@ -387,7 +390,150 @@ Event::List Calendar::sortEvents( Event::List *eventList,
   }
 
   return eventListSorted;
+}
 
+Event::List Calendar::sortEventsForDate( Event::List *eventList,
+                                         const QDate &date,
+                                         const KDateTime::Spec &timeSpec,
+                                         EventSortField sortField,
+                                         SortDirection sortDirection )
+{
+  Event::List eventListSorted;
+  Event::List tempList;
+  Event::List alphaList;
+  Event::List::Iterator sortIt;
+  Event::List::Iterator eit;
+
+  switch( sortField ) {
+  case EventSortStartDate:
+    alphaList = sortEvents( eventList, EventSortSummary, sortDirection );
+    for ( eit = alphaList.begin(); eit != alphaList.end(); ++eit ) {
+      if ( (*eit)->allDay() ) {
+        tempList.append( *eit );
+        continue;
+      }
+      sortIt = eventListSorted.begin();
+      if ( sortDirection == SortDirectionAscending ) {
+        while ( sortIt != eventListSorted.end() ) {
+          if ( !(*eit)->recurs() ) {
+            if ( (*eit)->dtStart().time() >= (*sortIt)->dtStart().time() ) {
+              ++sortIt;
+            } else {
+              break;
+            }
+          } else {
+            if ( (*eit)->recursOn( date, timeSpec ) ) {
+              if ( (*eit)->dtStart().time() >= (*sortIt)->dtStart().time() ) {
+                ++sortIt;
+              } else {
+                break;
+              }
+            } else {
+              ++sortIt;
+            }
+          }
+        }
+      } else { // descending
+        while ( sortIt != eventListSorted.end() ) {
+          if ( !(*eit)->recurs() ) {
+            if ( (*eit)->dtStart().time() < (*sortIt)->dtStart().time() ) {
+              ++sortIt;
+            } else {
+              break;
+            }
+          } else {
+            if ( (*eit)->recursOn( date, timeSpec ) ) {
+              if ( (*eit)->dtStart().time() < (*sortIt)->dtStart().time() ) {
+                ++sortIt;
+              } else {
+                break;
+              }
+            } else {
+              ++sortIt;
+            }
+          }
+        }
+      }
+      eventListSorted.insert( sortIt, *eit );
+    }
+    if ( sortDirection == SortDirectionAscending ) {
+      // Prepend the list of all-day Events
+      tempList += eventListSorted;
+      eventListSorted = tempList;
+    } else {
+      // Append the list of all-day Events
+      eventListSorted += tempList;
+    }
+    break;
+
+  case EventSortEndDate:
+    alphaList = sortEvents( eventList, EventSortSummary, sortDirection );
+    for ( eit = alphaList.begin(); eit != alphaList.end(); ++eit ) {
+      if ( (*eit)->hasEndDate() ) {
+        sortIt = eventListSorted.begin();
+        if ( sortDirection == SortDirectionAscending ) {
+          while ( sortIt != eventListSorted.end() ) {
+            if ( !(*eit)->recurs() ) {
+              if ( (*eit)->dtEnd().time() >= (*sortIt)->dtEnd().time() ) {
+                ++sortIt;
+              } else {
+                break;
+              }
+            } else {
+              if ( (*eit)->recursOn( date, timeSpec ) ) {
+                if ( (*eit)->dtEnd().time() >= (*sortIt)->dtEnd().time() ) {
+                  ++sortIt;
+                } else {
+                  break;
+                }
+              } else {
+                ++sortIt;
+              }
+            }
+          }
+        } else { // descending
+          while ( sortIt != eventListSorted.end() ) {
+            if ( !(*eit)->recurs() ) {
+              if ( (*eit)->dtEnd().time() < (*sortIt)->dtEnd().time() ) {
+                ++sortIt;
+              } else {
+                break;
+              }
+            } else {
+              if ( (*eit)->recursOn( date, timeSpec ) ) {
+                if ( (*eit)->dtEnd().time() < (*sortIt)->dtEnd().time() ) {
+                  ++sortIt;
+                } else {
+                  break;
+                }
+              } else {
+                ++sortIt;
+              }
+            }
+          }
+        }
+      } else {
+        // Keep a list of the Events without End DateTimes
+        tempList.append( *eit );
+      }
+      eventListSorted.insert( sortIt, *eit );
+    }
+    if ( sortDirection == SortDirectionAscending ) {
+      // Prepend the list of Events without End DateTimes
+      tempList += eventListSorted;
+      eventListSorted = tempList;
+    } else {
+      // Append the list of Events without End DateTimes
+      eventListSorted += tempList;
+    }
+    break;
+
+  default:
+    eventListSorted = sortEvents( eventList, sortField, sortDirection );
+    break;
+  }
+
+  return eventListSorted;
 }
 
 Event::List Calendar::events( const QDate &date,
@@ -790,6 +936,16 @@ Journal::List Calendar::journals( const QDate &date )
   return el;
 }
 
+void Calendar::beginBatchAdding()
+{
+  emit batchAddingBegins();
+}
+
+void Calendar::endBatchAdding()
+{
+  emit batchAddingEnds();
+}
+
 // When this is called, the to-dos have already been added to the calendar.
 // This method is only about linking related to-dos.
 void Calendar::setupRelations( Incidence *forincidence )
@@ -816,8 +972,16 @@ void Calendar::setupRelations( Incidence *forincidence )
     Incidence *parent = incidence( forincidence->relatedToUid() );
     if ( parent ) {
       // Found it
-      forincidence->setRelatedTo( parent );
-      parent->addRelation( forincidence );
+
+      // look for hierarchy loops
+      if ( isAncestorOf( forincidence, parent ) ) {
+        forincidence->setRelatedToUid( QString() );
+        kWarning() << "hierarchy loop beetween " << forincidence->uid() << " and " << parent->uid();
+      } else {
+        forincidence->setRelatedTo( parent );
+        parent->addRelation( forincidence );
+      }
+
     } else {
       // Not found, put this in the mOrphans list
       // Note that the mOrphans dict might contain multiple entries with the
@@ -894,6 +1058,29 @@ void Calendar::removeRelations( Incidence *incidence )
       }
     }
   }
+
+  // Make sure the deleted incidence doesn't relate to a non-deleted incidence,
+  // since that would cause trouble in CalendarLocal::close(), as the deleted
+  // incidences are destroyed after the non-deleted incidences. The destructor
+  // of the deleted incidences would then try to access the already destroyed
+  // non-deleted incidence, which would segfault.
+  //
+  // So in short: Make sure dead incidences don't point to alive incidences
+  // via the relation.
+  //
+  // This crash is tested in CalendarLocalTest::testRelationsCrash().
+  incidence->setRelatedTo( 0 );
+}
+
+bool Calendar::isAncestorOf( Incidence *ancestor, Incidence *incidence )
+{
+  if ( !incidence || incidence->relatedToUid().isEmpty() ) {
+    return false;
+  } else if ( incidence->relatedToUid() == ancestor->uid() ) {
+    return true;
+  } else {
+    return isAncestorOf( ancestor, this->incidence( incidence->relatedToUid() ) );
+  }
 }
 
 void Calendar::CalendarObserver::calendarModified( bool modified, Calendar *calendar )
@@ -1079,17 +1266,20 @@ void Calendar::appendRecurringAlarms( Alarm::List &alarms,
                                       const KDateTime &to )
 {
   KDateTime dt;
-  bool endOffsetValid = false;
   Duration endOffset( 0 );
+  bool endOffsetValid = false;
   Duration period( from, to );
 
+  Event *e = static_cast<Event *>( incidence );
+  Todo *t = static_cast<Todo *>( incidence );
+
   Alarm::List alarmlist = incidence->alarms();
   for ( int i = 0, iend = alarmlist.count();  i < iend;  ++i ) {
     Alarm *a = alarmlist[i];
     if ( a->enabled() ) {
       if ( a->hasTime() ) {
         // The alarm time is defined as an absolute date/time
-        dt = a->nextRepetition( from.addSecs(-1) );
+        dt = a->nextRepetition( from.addSecs( -1 ) );
         if ( !dt.isValid() || dt > to ) {
           continue;
         }
@@ -1103,27 +1293,46 @@ void Calendar::appendRecurringAlarms( Alarm::List &alarms,
         } else if ( a->hasEndOffset() ) {
           offset = a->endOffset();
           if ( !endOffsetValid ) {
-            endOffset = Duration( incidence->dtStart(), incidence->dtEnd() );
-            endOffsetValid = true;
+            if ( incidence->type() == "Event" ) {
+              endOffset = Duration( e->dtStart(), e->dtEnd() );
+              endOffsetValid = true;
+            } else if ( incidence->type() == "Todo" &&
+                        t->hasStartDate() && t->hasDueDate() ) {
+              endOffset = Duration( t->dtStart(), t->dtEnd() );
+              endOffsetValid = true;
+            }
           }
         }
 
         // Find the incidence's earliest alarm
-        KDateTime alarmStart =
-          offset.end( a->hasEndOffset() ? incidence->dtEnd() : incidence->dtStart() );
-//        KDateTime alarmStart = incidence->dtStart().addSecs( offset );
-        if ( alarmStart > to ) {
+        KDateTime alarmStart;
+        if ( incidence->type() == "Event" ) {
+          alarmStart =
+            offset.end( a->hasEndOffset() ? e->dtEnd() : e->dtStart() );
+        } else if ( incidence->type() == "Todo" ) {
+          alarmStart =
+            offset.end( a->hasEndOffset() ? t->dtDue() : t->dtStart() );
+        }
+
+        if ( alarmStart.isValid() && alarmStart > to ) {
           continue;
         }
-        KDateTime baseStart = incidence->dtStart();
-        if ( from > alarmStart ) {
+
+        KDateTime baseStart;
+        if ( incidence->type() == "Event" ) {
+          baseStart = e->dtStart();
+        } else if ( incidence->type() == "Todo" ) {
+          baseStart = t->dtDue();
+        }
+        if ( alarmStart.isValid() && from > alarmStart ) {
           alarmStart = from;   // don't look earlier than the earliest alarm
           baseStart = (-offset).end( (-endOffset).end( alarmStart ) );
         }
 
-        // Adjust the 'alarmStart' date/time and find the next recurrence at or after it.
-        // Treate the two offsets separately in case one is daily and the other not.
-        dt = incidence->recurrence()->getNextDateTime( baseStart.addSecs(-1) );
+        // Adjust the 'alarmStart' date/time and find the next recurrence
+        // at or after it. Treat the two offsets separately in case one
+        // is daily and the other not.
+        dt = incidence->recurrence()->getNextDateTime( baseStart.addSecs( -1 ) );
         if ( !dt.isValid() ||
              ( dt = endOffset.end( offset.end( dt ) ) ) > to ) // adjust 'dt' to get the alarm time
         {
@@ -1132,8 +1341,8 @@ void Calendar::appendRecurringAlarms( Alarm::List &alarms,
             continue;
           }
 
-          // The alarm has repetitions, so check whether repetitions of previous
-          // recurrences fall within the time period.
+          // The alarm has repetitions, so check whether repetitions of
+          // previous recurrences fall within the time period.
           bool found = false;
           Duration alarmDuration = a->duration();
           for ( KDateTime base = baseStart;
@@ -1143,8 +1352,9 @@ void Calendar::appendRecurringAlarms( Alarm::List &alarms,
               break;  // this recurrence's last repetition is too early, so give up
             }
 
-            // The last repetition of this recurrence is at or after 'alarmStart' time.
-            // Check if a repetition occurs between 'alarmStart' and 'to'.
+            // The last repetition of this recurrence is at or after
+            // 'alarmStart' time. Check if a repetition occurs between
+            // 'alarmStart' and 'to'.
             int snooze = a->snoozeTime().value();   // in seconds or days
             if ( a->snoozeTime().isDaily() ) {
               Duration toFromDuration( dt, base );
