@@ -1,7 +1,7 @@
 /*  -*- c++ -*-
     kmime_headers.h
 
-    KMime, the KDE internet mail/usenet news message library.
+    KMime, the KDE Internet mail/usenet news message library.
     Copyright (c) 2001-2002 the KMime authors.
     See file AUTHORS for details
     Copyright (c) 2006 Volker Krause <vkrause@kde.org>
@@ -68,20 +68,26 @@ enum contentCategory {
   CCalternativePart
 };
 
+/**
+  Various possible values for the "Content-Transfer-Encoding" header.
+*/
 enum contentEncoding {
-  CE7Bit,
-  CE8Bit,
-  CEquPr,
-  CEbase64,
-  CEuuenc,
-  CEbinary
+  CE7Bit,              ///< 7bit
+  CE8Bit,              ///< 8bit
+  CEquPr,              ///< quoted-printable
+  CEbase64,            ///< base64
+  CEuuenc,             ///< uuencode
+  CEbinary             ///< binary
 };
 
+/**
+  Various possible values for the "Content-Disposition" header.
+*/
 enum contentDisposition {
-  CDInvalid,
-  CDinline,
-  CDattachment,
-  CDparallel
+  CDInvalid,           ///< Default, invalid value
+  CDinline,            ///< inline
+  CDattachment,        ///< attachment
+  CDparallel           ///< parallel (invalid, do not use)
 };
 
 //often used charset
@@ -580,6 +586,7 @@ class KMIME_EXPORT SingleIdent : public Ident
 {
   //@cond PRIVATE
   kmime_mk_trivial_ctor( SingleIdent )
+  kmime_mk_dptr_ctor( SingleIdent )
   //@endcond
   public:
     /**
@@ -712,6 +719,12 @@ class KMIME_EXPORT Parametrized : public Structured
     */
     QString parameter( const QString &key ) const;
 
+    /**
+      @return true if a parameter with the given @p key exists.
+      @since 4.5
+    */
+    bool hasParameter( const QString &key ) const;
+
     /**
       Sets the parameter @p key to @p value.
       @param key The parameter name.
@@ -894,10 +907,23 @@ class KMIME_EXPORT ContentTransferEncoding : public Generics::Token
     */
     void setEncoding( contentEncoding e );
 
+    /**
+      Returns whether the Content containing this header is already decoded.
+    */
+    // KDE5: rename to isDecoded().
     bool decoded() const;
 
+    /**
+      Set whether the Content containing this header is already decoded.
+      For instance, if you fill your Content with already-encoded base64 data,
+      you will want to setDecoded( false ).
+    */
     void setDecoded( bool decoded = true );
 
+    /**
+      Returns whether the Content containing this header needs to be encoded
+      (i.e., if decoded() is true and encoding() is base64 or quoted-printable).
+    */
     bool needToEncode() const;
 
   protected:
@@ -949,12 +975,22 @@ class KMIME_EXPORT MessageID : public Generics::SingleIdent
     void generate( const QByteArray &fqdn );
 };
 
+class ContentIDPrivate;
+
 /**
   Represents a "Content-ID" header.
 */
 class KMIME_EXPORT ContentID : public Generics::SingleIdent
 {
+  //@cond PRIVATE
   kmime_mk_trivial_ctor_with_name( ContentID )
+  kmime_mk_dptr_ctor( ContentID )
+  //@endcond
+
+  protected:
+    bool parse( const char* &scursor, const char *const send, bool isCRLF=false );
+  private:
+    Q_DECLARE_PRIVATE(ContentID)
 };
 
 /**
