@@ -25,6 +25,7 @@
 #include "kmime_util_p.h"
 #include "kmime_header_parsing.h"
 #include "kmime_charfreq.h"
+#include "kmime_warning.h"
 
 #include <config-kmime.h>
 #include <kdefakes.h> // for strcasestr
@@ -42,6 +43,8 @@
 #include <time.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <boost/concept_check.hpp>
+#include "kmime_codecs.h"
 
 using namespace KMime;
 
@@ -49,6 +52,8 @@ namespace KMime {
 
 QList<QByteArray> c_harsetCache;
 QList<QByteArray> l_anguageCache;
+QString f_allbackCharEnc;
+bool u_seOutlookEncoding = false;
 
 QByteArray cachedCharset( const QByteArray &name )
 {
@@ -173,6 +178,27 @@ const uchar eTextMap[16] = {
   0x7F, 0xFF, 0xFF, 0xE0
 };
 
+void setFallbackCharEncoding(const QString& fallbackCharEnc)
+{
+  f_allbackCharEnc = fallbackCharEnc;
+}
+
+QString fallbackCharEncoding()
+{
+  return f_allbackCharEnc;
+}
+
+void setUseOutlookAttachmentEncoding( bool violateStandard )
+{
+  u_seOutlookEncoding = violateStandard;
+}
+
+bool useOutlookAttachmentEncoding()
+{
+  return u_seOutlookEncoding;
+}
+
+
 QString decodeRFC2047String( const QByteArray &src, QByteArray &usedCS,
                              const QByteArray &defaultCS, bool forceCS )
 {
@@ -218,8 +244,15 @@ QString decodeRFC2047String( const QByteArray &src, QByteArray &usedCS,
       ++scursor;
     }
   }
-
-  return QString::fromUtf8(result);
+  // If there are any chars that couldn't be decoded in UTF-8,
+  //  use the fallback charset if it exists
+  const QString tryUtf8 = QString::fromUtf8( result );
+  if ( tryUtf8.contains( 0xFFFD ) && !f_allbackCharEnc.isEmpty() ) {
+    QTextCodec* codec = KGlobal::charsets()->codecForName( f_allbackCharEnc );
+    return codec->toUnicode( result );
+  } else {
+    return tryUtf8;
+  }
 }
 
 QString decodeRFC2047String( const QByteArray &src )
@@ -285,7 +318,7 @@ QByteArray encodeRFC2047String( const QString &src, const QByteArray &charset,
     for ( int x=end; x<encoded8Bit.length(); x++ ) {
       if ( ( (signed char)encoded8Bit[x]<0) || ( encoded8Bit[x] == '\033' ) ||
            ( addressHeader && ( strchr("\"()<>@,.;:\\[]=",encoded8Bit[x]) != 0 ) ) ) {
-        end = encoded8Bit.length();     // we found another non-ascii word
+        end = x;     // we found another non-ascii word
 
         while ( ( end < encoded8Bit.length() ) && ( encoded8Bit[end] != ' ' ) ) {
           // we encode complete words
@@ -337,6 +370,112 @@ QByteArray encodeRFC2047String( const QString &src, const QByteArray &charset,
   return result;
 }
 
+
+//-----------------------------------------------------------------------------
+QByteArray encodeRFC2231String( const QString& str, const QByteArray& charset )
+{
+  if ( str.isEmpty() )
+    return QByteArray();
+
+  
+  const QTextCodec *codec = KGlobal::charsets()->codecForName( charset );
+  QByteArray latin;
+  if ( charset == "us-ascii" )
+    latin = str.toAscii();
+  else if ( codec )
+    latin = codec->fromUnicode( str );
+  else
+    latin = str.toLocal8Bit();
+
+  char *l;
+  for ( l = latin.data(); *l; ++l ) {
+    if ( ( ( *l & 0xE0 ) == 0 ) || ( *l & 0x80 ) )
+      // *l is control character or 8-bit char
+      break;
+  }
+  if ( !*l )
+    return latin;
+
+  QByteArray result = charset + "''";
+  for ( l = latin.data(); *l; ++l ) {
+    bool needsQuoting = ( *l & 0x80 ) || ( *l == '%' );
+    if( !needsQuoting ) {
+      const QByteArray especials = "()<>@,;:\"/[]?.= \033";
+      int len = especials.length();
+      for ( int i = 0; i < len; i++ )
+        if ( *l == especials[i] ) {
+          needsQuoting = true;
+          break;
+        }
+    }
+    if ( needsQuoting ) {
+      result += '%';
+      unsigned char hexcode;
+      hexcode = ( ( *l & 0xF0 ) >> 4 ) + 48;
+      if ( hexcode >= 58 )
+        hexcode += 7;
+      result += hexcode;
+      hexcode = ( *l & 0x0F ) + 48;
+      if ( hexcode >= 58 )
+        hexcode += 7;
+      result += hexcode;
+    } else {
+      result += *l;
+    }
+  }
+  return result;
+}
+
+
+//-----------------------------------------------------------------------------
+QString decodeRFC2231String( const QByteArray &str, QByteArray &usedCS, const QByteArray &defaultCS,
+  bool forceCS )
+{
+  int p = str.indexOf('\'');
+  if (p < 0) return KGlobal::charsets()->codecForName( defaultCS )->toUnicode( str );
+
+  
+  QByteArray charset = str.left(p);
+
+  QByteArray st = str.mid( str.lastIndexOf('\'') + 1 );
+  
+  char ch, ch2;
+  p = 0;
+  while (p < (int)st.length())
+  {
+    if (st.at(p) == 37)
+    {
+      // Only try to decode the percent-encoded character if the percent sign
+      // is really followed by two other characters, see testcase at bug 163024
+      if ( p + 2 < st.length() ) {
+        ch = st.at(p+1) - 48;
+        if (ch > 16)
+          ch -= 7;
+        ch2 = st.at(p+2) - 48;
+        if (ch2 > 16)
+          ch2 -= 7;
+        st[p] = ch * 16 + ch2;
+        st.remove( p+1, 2 );
+      }
+    }
+    p++;
+  }
+  kDebug() << "Got pre-decoded:" << st;
+  QString result;
+  const QTextCodec * charsetcodec = KGlobal::charsets()->codecForName( charset );
+  if ( !charsetcodec || forceCS )
+    charsetcodec = KGlobal::charsets()->codecForName( defaultCS );
+
+  usedCS = charsetcodec->name();
+  return charsetcodec->toUnicode( st );
+}
+
+QString decodeRFC2231String( const QByteArray &src )
+{
+  QByteArray usedCS;
+  return decodeRFC2231String( src, usedCS, "utf-8", false );
+}
+
 QByteArray uniqueString()
 {
   static char chars[] = "0123456789abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
@@ -384,11 +523,23 @@ QByteArray unfoldHeader( const QByteArray &header )
     }
     // find the first non-space after the line-break
     while ( foldEnd <= header.length() - 1 ) {
-      if ( !QChar( header[foldEnd] ).isSpace() ) {
+      if ( QChar( header[foldEnd] ).isSpace() ) {
+        ++foldEnd;
+      }
+      else if ( foldEnd > 0 && header[foldEnd - 1] == '\n' &&
+                header[foldEnd] == '=' && foldEnd + 2 < header.length() &&
+                ( ( header[foldEnd + 1] == '0' &&
+                    header[foldEnd + 2] == '9' ) ||
+                  ( header[foldEnd + 1] == '2' &&
+                    header[foldEnd + 2] == '0' ) ) ) {
+        // bug #86302: malformed header continuation starting with =09/=20
+        foldEnd += 3;
+      }
+      else {
         break;
       }
-      ++foldEnd;
     }
+
     result += header.mid( pos, foldBegin - pos );
     if ( foldEnd < header.length() -1 )
       result += ' ';
@@ -398,6 +549,62 @@ QByteArray unfoldHeader( const QByteArray &header )
   return result;
 }
 
+int findHeaderLineEnd( const QByteArray &src, int &dataBegin, bool *folded )
+{
+  int end = dataBegin;
+  int len = src.length() - 1;
+
+  if ( folded )
+    *folded = false;
+
+  if ( dataBegin < 0 ) {
+    // Not found
+    return -1;
+  }
+
+  if ( dataBegin > len ) {
+    // No data available
+    return len + 1;
+  }
+
+  // If the first line contains nothing, but the next line starts with a space
+  // or a tab, that means a stupid mail client has made the first header field line
+  // entirely empty, and has folded the rest to the next line(s).
+  if ( src.at(end) == '\n' && end + 1 < len &&
+       ( src[end+1] == ' ' || src[end+1] == '\t' ) ) {
+
+    // Skip \n and first whitespace
+    dataBegin += 2;
+    end += 2;
+  }
+
+  if ( src.at(end) != '\n' ) {  // check if the header is not empty
+    while ( true ) {
+      end = src.indexOf( '\n', end + 1 );
+      if ( end == -1 || end == len ) {
+        // end of string
+        break;
+      }
+      else if ( src[end+1] == ' ' || src[end+1] == '\t' ||
+                ( src[end+1] == '=' && end+3 <= len &&
+                  ( ( src[end+2] == '0' && src[end+3] == '9' ) ||
+                    ( src[end+2] == '2' && src[end+3] == '0' ) ) ) ) {
+        // next line is header continuation or starts with =09/=20 (bug #86302)
+        if ( folded )
+          *folded = true;
+      } else {
+        // end of header (no header continuation)
+        break;
+      }
+    }
+  }
+
+  if ( end < 0 ) {
+    end = len + 1; //take the rest of the string
+  }
+  return end;
+}
+
 int indexOfHeader( const QByteArray &src, const QByteArray &name, int &end, int &dataBegin, bool *folded )
 {
   QByteArray n = name;
@@ -423,28 +630,7 @@ int indexOfHeader( const QByteArray &src, const QByteArray &name, int &end, int
     if ( src.at( dataBegin ) == ' ' ) {
       ++dataBegin;
     }
-    end = dataBegin;
-    int len = src.length() - 1;
-    if ( folded )
-      *folded = false;
-
-    if ( src.at(end) != '\n' ) {  // check if the header is not empty
-      while ( true ) {
-        end = src.indexOf( '\n', end + 1 );
-        if ( end == -1 || end == len ||
-             ( src[end+1] != ' ' && src[end+1] != '\t' ) ) {
-          //break if we reach the end of the string, honor folded lines
-          break;
-        } else {
-          if ( folded )
-            *folded = true;
-        }
-      }
-    }
-
-    if ( end < 0 ) {
-      end = len + 1; //take the rest of the string
-    }
+    end = findHeaderLineEnd( src, dataBegin, folded );
     return begin;
 
   } else {
@@ -511,6 +697,12 @@ QByteArray CRLFtoLF( const QByteArray &s )
   return ret;
 }
 
+QByteArray CRLFtoLF( const char *s )
+{
+  QByteArray ret = s;
+  return CRLFtoLF( ret );
+}
+
 QByteArray LFtoCRLF( const QByteArray &s )
 {
   QByteArray ret = s;
@@ -518,6 +710,12 @@ QByteArray LFtoCRLF( const QByteArray &s )
   return ret;
 }
 
+QByteArray LFtoCRLF( const char *s )
+{
+  QByteArray ret = s;
+  return LFtoCRLF( ret );
+}
+
 namespace {
 template < typename T > void removeQuotesGeneric( T & str )
 {
@@ -546,14 +744,33 @@ void removeQuots( QString &str )
   removeQuotesGeneric( str );
 }
 
-void addQuotes( QByteArray &str, bool forceQuotes )
+//
+// The next two helper function are just functions that return the ASCII char of
+// a string or an array. This is only there to facilitate writing addQuotes_impl()
+// without code duplication
+//
+
+static char getCharFromQByteArray( const QByteArray &array, int index )
+{
+  return array.at( index );
+}
+
+static char getCharFromQString( const QString &string, int index )
+{
+  return string.at( index ).toAscii();
+}
+
+template<class StringType>
+void addQuotes_impl( StringType &str, bool forceQuotes,
+                     char (*convertFunction)( const StringType&, int ) )
 {
   bool needsQuotes=false;
   for ( int i=0; i < str.length(); i++ ) {
-    if ( strchr("()<>@,.;:[]=\\\"", str[i] ) != 0 ) {
+    const char cur = convertFunction( str, i );
+    if ( strchr("()<>@,.;:[]=\\\"", cur ) != 0 ) {
       needsQuotes = true;
     }
-    if ( str[i] == '\\' || str[i] == '\"' ) {
+    if ( cur == '\\' || cur == '\"' ) {
       str.insert( i, '\\' );
       i++;
     }
@@ -565,6 +782,16 @@ void addQuotes( QByteArray &str, bool forceQuotes )
   }
 }
 
+void addQuotes( QByteArray &str, bool forceQuotes )
+{
+  addQuotes_impl( str, forceQuotes, &getCharFromQByteArray );
+}
+
+void addQuotes( QString &str, bool forceQuotes )
+{
+  addQuotes_impl( str, forceQuotes, &getCharFromQString );
+}
+
 KMIME_EXPORT QString balanceBidiState( const QString &input )
 {
   const int LRO = 0x202D;
